'===============================================================================
'Library for the Nokia 3310 display
'This is v2.0 with support for printing bitmap images.
'This is the second of at least two .bas files that must be used together.
'Please see 3310init.bas for instructions.
'exersizze@gmail.com
'http://www.avrproject.ru
'===============================================================================

Sub D3310reset
   D3310re = 0
   Waitus 250
   D3310re = 1
End Sub

Sub D3310init
   D3310ce = 0                                              'Display data input enable
   D3310dc = 0                                              'Command mode
   D3310commandout &H21                                     'Extended command mode, horizontal addressing
   D3310commandout &HC0                                     'Middle contrast    (c8)
   D3310commandout &H06                                     'Temperature setting
   D3310commandout &H13                                     'Bias 1:48
   D3310commandout &H20                                     'Normal command mode, horizontal addressing
   D3310commandout &H0C                                     'Uninverted Screen
   D3310ce = 1                                              'Display data input disable
End Sub

Sub D3310clear

   Local D3310pixelloop As Word                             'Loop variable for the pixels

   D3310ce = 0                                              'Display data input enable
   D3310dc = 0                                              'Command mode
   D3310commandout &H20                                     'Normal command mode, horizontal addressing
   D3310commandout &H80                                     'Set RAM X address to 0
   D3310commandout &H40                                     'Set RAM Y address to 0
   D3310dc = 1                                              'Go to data mode
   For D3310pixelloop = 0 To 503                            'Send 504 blank pixels
      D3310dataout &H00
   Next D3310pixelloop
   D3310ce = 1                                              'Display data input disable
End Sub

Sub D3310invert
   D3310commandout &H0D                                     'Inverted screen
End Sub

Sub D3310uninvert
   D3310commandout &H0C                                     'Uninverted screen
End Sub

Sub D3310commandout(byval D3310value As Byte)
   D3310ce = 0                                              'Display data input enable
   D3310dc = 0                                              'Command mode
   Spiout D3310value , 1
   D3310ce = 1                                              'Display data input disable
End Sub

Sub D3310dataout(byval D3310value As Byte)
   D3310ce = 0                                              'Display data input enable
   D3310dc = 1                                              'Data mode
   Spiout D3310value , 1
   D3310ce = 1                                              'Display data input disable
End Sub

Sub D3310position(byval D3310x As Byte , Byval D3310y As Byte)       'Y is the row (0-5), X is the column (0-13)

   Local D3310byte As Byte                                  'Command or data output

   D3310commandout &H20                                     'Horizontal Addressing

   Waitus 250                                               'Wait for the CE to settle

   D3310ce = 0                                              'Display data input enable
   D3310dc = 0                                              'Command mode

   D3310byte = D3310x * &H06                                'Send horizontal address
   D3310byte = D3310byte + &H80
   Spiout D3310byte , 1

   D3310byte = D3310y + &H40                                'Send vertical address
   Spiout D3310byte , 1

   D3310ce = 1                                              'Display data input disable

End Sub

Sub D3310bmpout                                             'You must Restore D3310bmp* before you call D3310bmpout

   Local D3310pixelloop As Word                             'Loop variable for the pixels
   Local D3310column As Byte                                'Display column output

   D3310ce = 0                                              'Display data input enable
   D3310dc = 0                                              'Command mode
   D3310commandout &H20                                     'Normal command mode, horizontal addressing
   D3310commandout &H80                                     'Set RAM X address to 0
   D3310commandout &H40                                     'Set RAM Y address to 0
   D3310dc = 1                                              'Go to data mode

   For D3310pixelloop = 0 To 503                            'Read and print the 504 pixel columns
      Read D3310column
      Call D3310dataout(d3310column)
   Next D3310pixelloop
   D3310ce = 1                                              'Display data input disable

End Sub

Sub D3310print(byval D3310string As String * 14)

   Local D3310length As Byte                                'Holds the length of D3310string
   Local D3310character As String * 1                       'Holds each consecutive character in turn during display output
   Local D3310stringloop As Byte                            'Loop variable for the string
   Local D3310columnloop As Byte                            'Loop variable for the columns
   Local D3310column As Byte                                'Display column output

   D3310length = Len(d3310string)                           'Establish the number of characters

   For D3310stringloop = 1 To D3310length                   'Loop through the string

      D3310character = Mid(d3310string , D3310stringloop , 1)       'Get the current character

      Select Case D3310character                            'Position the pointer to the correct character in the Eeprom and read the first byte

         Case "0" : Restore Zero
         Case "1" : Restore One
         Case "2" : Restore Two
         Case "3" : Restore Three
         Case "4" : Restore Four
         Case "5" : Restore Five
         Case "6" : Restore Six
         Case "7" : Restore Seven
         Case "8" : Restore Eight
         Case "9" : Restore Nine
         Case "A" : Restore aA
         Case "B" : Restore aB
         Case "C" : Restore aC
         Case "D" : Restore aD
         Case "E" : Restore aE
         Case "F" : Restore aF
         Case "G" : Restore aG
         Case "H" : Restore aH
         Case "I" : Restore aI
         Case "J" : Restore aJ
         Case "K" : Restore aK
         Case "L" : Restore aL
         Case "M" : Restore aM
         Case "N" : Restore aN
         Case "O" : Restore aO
         Case "P" : Restore aP
         Case "Q" : Restore aQ
         Case "R" : Restore aR
         Case "S" : Restore aaS
         Case "T" : Restore aaT
         Case "U" : Restore aU
         Case "V" : Restore aV
         Case "W" : Restore aW
         Case "X" : Restore aX
         Case "Y" : Restore aY
         Case "Z" : Restore aZ
         Case "a" : Restore aA1
         Case "b" : Restore aB1
         Case "c" : Restore aC1
         Case "d" : Restore aD1
         Case "e" : Restore aE1
         Case "f" : Restore aF1
         Case "g" : Restore aG1
         Case "h" : Restore aH1
         Case "i" : Restore aI1
         Case "j" : Restore aJ1
         Case "k" : Restore aK1
         Case "l" : Restore aL1
         Case "m" : Restore aM1
         Case "n" : Restore aN1
         Case "o" : Restore aO1
         Case "p" : Restore aP1
         Case "q" : Restore aQ1
         Case "r" : Restore aR1
         Case "s" : Restore aS1
         Case "t" : Restore aT1
         Case "u" : Restore aU1
         Case "v" : Restore aV1
         Case "w" : Restore aW1
         Case "x" : Restore aX1
         Case "y" : Restore aY1
         Case "z" : Restore aZ1


         Case "" : Restore aA
         Case "" : Restore aA1

         Case "" : Restore Be
         Case "" : Restore Be1

         Case "" : Restore Ve
         Case "" : Restore Ve1

         Case "" : Restore Ge
         Case "" : Restore Ge1

         Case "" : Restore De
         Case "" : Restore De1

         Case "" : Restore aE
         Case "" : Restore aE1

         Case "" : Restore Jo
         Case "" : Restore Jo1

         Case "" : Restore Je
         Case "" : Restore Je1

         Case "" : Restore Ze
         Case "" : Restore Ze1

         Case "" : Restore Ji
         Case "" : Restore Ji1

         Case "" : Restore Ij
         Case "" : Restore Ij1

         Case "" : Restore aK
         Case "" : Restore Ka

         Case "" : Restore Le
         Case "" : Restore Le1

         Case "" : Restore aM
         Case "" : Restore Me

         Case "" : Restore aH
         Case "" : Restore Ne

         Case "" : Restore aO
         Case "" : Restore aO1


         Case "" : Restore aPe
         Case "" : Restore aPe1

         Case "" : Restore aP
         Case "" : Restore aP1

         Case "" : Restore aC
         Case "" : Restore aC1

         Case "" : Restore aaT
         Case "" : Restore Te

         Case "" : Restore Yu
         Case "" : Restore aY1

         Case "" : Restore aX
         Case "" : Restore aX1


         Case "" : Restore Che
         Case "" : Restore Che1

         Case "" : Restore Ce
         Case "" : Restore Ce1


         Case "" : Restore aFe
         Case "" : Restore aFe1

         Case "" : Restore She
         Case "" : Restore She1

         Case "" : Restore Sche
         Case "" : Restore Sche1

         Case "" : Restore aB1
         Case "" : Restore Myag

         Case "" : Restore Ll3
         Case "" : Restore Tverd1

         Case "" : Restore Bi
         Case "" : Restore Bi1

         Case "" : Restore Ye
         Case "" : Restore Ye1

         Case "" : Restore Ju
         Case "" : Restore Ju1

         Case "" : Restore Ya
         Case "" : Restore Ya1





         Case " " : Restore Spacesign
         Case "-" : Restore Dash
         Case "." : Restore Dot
         Case ":" : Restore Colon
         Case ">" : Restore Arrowright
         Case "" : Restore Hh1
         Case "@" : Restore Hh2
         Case "&" : Restore Ll1
         Case "'" : Restore Hh4
         Case "<" : Restore Arrowleft
         Case "*" : Restore Hh3
         Case "," : Restore Comma
         Case "$" : Restore Ll2
         Case "=" : Restore Equals
         Case "!" : Restore Exclamation
         Case "" : Restore Grid
         Case "#" : Restore Ll4
         Case "(" : Restore Leftparagraph
         Case "[" : Restore Leftsquareparagraph
         Case "%" : Restore Percent
         Case "+" : Restore Plus
         Case "^" : Restore Powerof
         Case "?" : Restore Question
         Case D3310quotation : Restore Quotation
         Case "]" : Restore Rightparagraph
         Case ")" : Restore Rightsquareparagraph
         Case ";" : Restore Semicolon
         Case "/" : Restore Slash
         Case "\" : Restore Backslash
         Case "_" : Restore Underscore

      End Select

      For D3310columnloop = 1 To 5                          'Read and print each column
         Read D3310column
         Call D3310dataout(d3310column)
      Next D3310columnloop

      D3310column = 0                                       'And print a blank column for inter-character spacing
      Call D3310dataout(d3310column)

   Next D3310stringloop

   D3310ce = 1                                              'Display data input disable

End Sub

'===============================================================================

' 
Hh1:
Data &H04 , &H04 , &H04 , &H04 , &H04
Hh2:
Data &H01 , &H01 , &H01 , &H01 , &H01
Hh3:
Data &H10 , &H10 , &H10 , &H10 , &H10
Hh4:
Data &H40 , &H40 , &H40 , &H40 , &H40

Ll1:
Data &H01 , &H01 , &H01 , &H01 , &H01
Ll2:
Data &H04 , &H04 , &H04 , &H04 , &H04
Ll3:
Data &H10 , &H10 , &H10 , &H10 , &H10
Ll4:
Data &H40 , &H40 , &H40 , &H40 , &H40







'ASCII data

' 

'
Be:
Data &H7F , &H49 , &H49 , &H49 , &H31
Be1:
Data &H7C , &H4A , &H4A , &H49 , &H31

'
Ve:
Data &H7F , &H49 , &H49 , &H49 , &H36
Ve1:
Data &H7C , &H54 , &H54 , &H28 , &H00

'
Ge:
Data &H7F , &H01 , &H01 , &H01 , &H03
Ge1:
Data &H7C , &H04 , &H04 , &H04 , &H0C

'
De:
Data &HE0 , &H51 , &H4F , &H41 , &HFF
De1:
Data &HE0 , &H54 , &H4C , &H44 , &HFC

'
Jo:
Data &H7C , &H55 , &H54 , &H55 , &H44
Jo1:
Data &H38 , &H55 , &H54 , &H55 , &H18
'
Je:
Data &H77 , &H08 , &H7F , &H08 , &H77
Je1:
Data &H6C , &H10 , &H7C , &H10 , &H6C

'
Ze:
Data &H41 , &H49 , &H49 , &H49 , &H36
Ze1:
Data &H44 , &H54 , &H54 , &H54 , &H28

'
Ji:
Data &H7F , &H10 , &H08 , &H04 , &H7F
Ji1:
Data &H7C , &H20 , &H10 , &H08 , &H7C

'
Ij:
Data &H7C , &H21 , &H12 , &H09 , &H7C
Ij1:
Data &H78 , &H42 , &H24 , &H12 , &H78

'
Ka:
Data &H7C , &H10 , &H28 , &H44 , &H00

'
Le:
Data &H20 , &H41 , &H3F , &H01 , &H7F
Le1:
Data &H20 , &H44 , &H3C , &H04 , &H7C

'
Me:
Data &H7C , &H08 , &H10 , &H08 , &H7C

'
Ne:
Data &H7C , &H10 , &H10 , &H10 , &H7C

'
aPe:
Data &H7F , &H01 , &H01 , &H01 , &H7F
aPe1:
Data &H7C , &H04 , &H04 , &H04 , &H7C


'T
Te:
Data &H04 , &H04 , &H7C , &H04 , &H04

'
Yu:
Data &H47 , &H28 , &H10 , &H08 , &H07

'
Che:
Data &H07 , &H08 , &H08 , &H08 , &H7F
Che1:
Data &H0C , &H10 , &H10 , &H10 , &H7C

'
aFe:
Data &H0C , &H12 , &H7F , &H12 , &H0C
aFe1:
Data &H18 , &H24 , &H7E , &H24 , &H18

'
Ce:
Data &H7F , &H40 , &H40 , &H40 , &HFF
Ce1:
Data &H7C , &H40 , &H40 , &H40 , &HFC


'
She:
Data &H7F , &H40 , &H7F , &H40 , &H7F
She1:
Data &H7C , &H40 , &H78 , &H40 , &H7C

'
Sche:
Data &H7F , &H40 , &H7F , &H40 , &HFF
Sche1:
Data &H7C , &H40 , &H78 , &H40 , &HFC

'
Myag:
Data &H7C , &H50 , &H50 , &H20 , &H00

'
Tverd:
Data &H01 , &H7F , &H48 , &H48 , &H30
Tverd1:
Data &H00 , &H07 , &H05 , &H07 , &H00

'
Bi:
Data &H00 , &H07 , &H00 , &H07 , &H00
Bi1:
Data &H7C , &H50 , &H20 , &H00 , &H7C

'
Ye:
Data &H22 , &H41 , &H49 , &H49 , &H3E
Ye1:
Data &H28 , &H44 , &H54 , &H54 , &H38


'
Ju:
Data &H7F , &H08 , &H3E , &H41 , &H3E
Ju1:
Data &H7C , &H10 , &H38 , &H44 , &H38


'
Ya:
Data &H46 , &H29 , &H19 , &H09 , &H7F
Ya1:
Data &H08 , &H54 , &H34 , &H14 , &H7C












'Digits
Zero:
Data &H3E , &H51 , &H49 , &H45 , &H3E
One:
Data &H00 , &H42 , &H7F , &H40 , &H00
Two:
Data &H42 , &H61 , &H51 , &H49 , &H46
Three:
Data &H22 , &H41 , &H49 , &H49 , &H36
Four:
Data &H18 , &H14 , &H12 , &H7F , &H10
Five:
Data &H27 , &H45 , &H45 , &H45 , &H39
Six:
Data &H3E , &H49 , &H49 , &H49 , &H32
Seven:
Data &H61 , &H11 , &H09 , &H05 , &H03
Eight:
Data &H36 , &H49 , &H49 , &H49 , &H36
Nine:
Data &H26 , &H49 , &H49 , &H49 , &H3E

'Upper-case
aA:
Data &H7E , &H09 , &H09 , &H09 , &H7E
aB:
Data &H7F , &H49 , &H49 , &H49 , &H36
aC:
Data &H3E , &H41 , &H41 , &H41 , &H22
aD:
Data &H7F , &H41 , &H41 , &H41 , &H3E
aE:
Data &H7F , &H49 , &H49 , &H49 , &H41
aF:
Data &H7F , &H09 , &H09 , &H09 , &H01
aG:
Data &H3E , &H41 , &H49 , &H49 , &H3A
aH:
Data &H7F , &H08 , &H08 , &H08 , &H7F
aI:
Data &H00 , &H41 , &H7F , &H41 , &H00
aJ:
Data &H20 , &H41 , &H41 , &H41 , &H3F
aK:
Data &H7F , &H08 , &H14 , &H22 , &H41
aL:
Data &H7F , &H40 , &H40 , &H40 , &H40
aM:
Data &H7F , &H02 , &H04 , &H02 , &H7F
aN:
Data &H7F , &H04 , &H08 , &H10 , &H7F
aO:
Data &H3E , &H41 , &H41 , &H41 , &H3E
aP:
Data &H7F , &H09 , &H09 , &H09 , &H06
aQ:
Data &H3E , &H41 , &H51 , &H61 , &H7E
aR:
Data &H7F , &H09 , &H19 , &H29 , &H46
aaS:
Data &H06 , &H49 , &H49 , &H49 , &H30
aaT:
Data &H01 , &H01 , &H7F , &H01 , &H01
aU:
Data &H3F , &H40 , &H40 , &H40 , &H3F
aV:
Data &H1F , &H20 , &H40 , &H20 , &H1F
aW:
Data &H7F , &H20 , &H10 , &H20 , &H7F
aX:
Data &H63 , &H14 , &H08 , &H14 , &H63
aY:
Data &H03 , &H04 , &H78 , &H04 , &H03
aZ:
Data &H61 , &H51 , &H49 , &H45 , &H43


'Lower-case
aA1:
Data &H20 , &H54 , &H54 , &H54 , &H78
aB1:
Data &H7F , &H48 , &H44 , &H44 , &H38
aC1:
Data &H38 , &H44 , &H44 , &H44 , &H28
aD1:
Data &H38 , &H44 , &H44 , &H48 , &H7F
aE1:
Data &H38 , &H54 , &H54 , &H54 , &H18
aF1:
Data &H00 , &H04 , &H7E , &H05 , &H00
aG1:
Data &H0C , &H52 , &H52 , &H4A , &H3C
aH1:
Data &H7F , &H08 , &H04 , &H04 , &H78
aI1:
Data &H00 , &H44 , &H7D , &H40 , &H00
aJ1:
Data &H00 , &H40 , &H44 , &H3D , &H00
aK1:
Data &H7F , &H20 , &H10 , &H28 , &H44
aL1:
Data &H00 , &H41 , &H7F , &H40 , &H00
aM1:
Data &H7C , &H04 , &H18 , &H04 , &H7C
aN1:
Data &H7C , &H08 , &H04 , &H04 , &H78
aO1:
Data &H38 , &H44 , &H44 , &H44 , &H38
aP1:
Data &H7C , &H14 , &H24 , &H24 , &H18
aQ1:
Data &H18 , &H24 , &H24 , &H14 , &H7C
aR1:
Data &H7C , &H04 , &H04 , &H08 , &H00
aS1:
Data &H08 , &H54 , &H54 , &H54 , &H20
aT1:
Data &H04 , &H3F , &H44 , &H40 , &H00
aU1:
Data &H3C , &H40 , &H40 , &H40 , &H7C
aV1:
Data &H1C , &H20 , &H40 , &H20 , &H1C
aW1:
Data &H3C , &H40 , &H30 , &H40 , &H3C
aX1:
Data &H44 , &H28 , &H10 , &H28 , &H44
aY1:
Data &H0C , &H50 , &H50 , &H48 , &H3C
aZ1:
Data &H44 , &H64 , &H54 , &H4C , &H44


'Special characters
Spacesign:
Data &H00 , &H00 , &H00 , &H00 , &H00
Dot:
Data &H00 , &H60 , &H60 , &H00 , &H00
Dash:
Data &H08 , &H08 , &H08 , &H08 , &H08
Colon:
Data &H00 , &H36 , &H36 , &H00 , &H00
Degree:
Data &H00 , &H07 , &H05 , &H07 , &H00
Leftsquareparagraph:
Data &H00 , &H7F , &H41 , &H41 , &H00
Rightsquareparagraph:
Data &H00 , &H41 , &H41 , &H7F , &H00
Grid:
Data &H55 , &H2A , &H55 , &H2A , &H55
Powerof:
Data &H00 , &H07 , &H00 , &H00 , &H00
Underscore:
Data &H40 , &H40 , &H40 , &H40 , &H40
Semicolon:
Data &H00 , &H56 , &H36 , &H00 , &H00
Arrowleft:
Data &H08 , &H1C , &H36 , &H63 , &H41
Arrowright:
Data &H41 , &H63 , &H36 , &H1C , &H08
Equals:
Data &H14 , &H14 , &H14 , &H14 , &H14
Question:
Data &H02 , &H01 , &H51 , &H09 , &H06
Alpha:
Data &H32 , &H49 , &H59 , &H51 , &H3E
Exclamation:
Data &H00 , &H00 , &H5F , &H00 , &H00
Quotation:
Data &H00 , &H07 , &H00 , &H07 , &H00
Hash:
Data &H14 , &H7F , &H14 , &H7F , &H14
Dollar:
Data &H24 , &H2A , &H7F , &H2A , &H12
Percent:
Data &H23 , &H13 , &H08 , &H64 , &H62
Ampersand:
Data &H36 , &H49 , &H55 , &H22 , &H50
Apostrophe:
Data &H07 , &H00 , &H00 , &H00 , &H00
Leftparagraph:
Data &H00 , &H1C , &H22 , &H41 , &H00
Rightparagraph:
Data &H00 , &H41 , &H22 , &H1C , &H00
Asterisc:
Data &H14 , &H08 , &H3E , &H08 , &H14
Plus:
Data &H08 , &H08 , &H3E , &H08 , &H08
Comma:
Data &H00 , &H00 , &H50 , &H30 , &H00
Slash:
Data &H20 , &H10 , &H08 , &H04 , &H02
Backslash:
Data &H02 , &H04 , &H08 , &H10 , &H20